


# This script is used to compute SPEI based on FAO penman monteith approach at EEA river subbasin level

library(s2dv)
library(s2dverification)
library(startR)
library(multiApply)
#library(zoo)
library(SPEI)
library(checkmate)

path.downloaded_data <- 'cds/'
path.intermediate_data <- 'folder/'

################################################################################################
################################## LOAD THE DATA: OBS ##########################################
################################################################################################

variables = c('tasmax','tasmin','prlr','tdps','sfcWind','ps','rsds')
current_dataset <- 'era5land'
retrieve <- FALSE

years_obs = 1950:2022
n_procs_obs <- 16

# European Domain
lat_min <- 25
lat_max <- 75
lon_min <- -15
lon_max <- 44

# Load monthly tas and transform

variable <- variables[1]
path_obs_tasmax=paste0(path.downloaded_data, 'era5land/monthly_mean/tasmax_f24h/$var$_$year$$month$.nc')

  data_tasmax_obs = Start(dataset = path_obs_tasmax,
                       var = variable,

                       year = paste0(years_obs),
                       month = 'all',

                       # lat = 'all',
                       # lon = values(lon_sel),

                       lat = values(list(lat_min,lat_max)), lat_reorder = Sort(decreasing = TRUE), lat_var = 'lat',
                       lon = values(list(lon_min,lon_max)), lon_reorder = CircularSort(-180,180), lon_var = 'lon',

                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       #lat_reorder = Sort(decreasing = F),
                       #lon_reorder = CircularSort(0, 360),
                       num_procs = n_procs_obs, 
                       retrieve = retrieve)

  names(dim(data_tasmax_obs)) <- c("dat","member","sdate","time","latitude","longitude")

# From K to degree C
  
  factor_tmean=273.15
  data_tasmax_obs <- data_tasmax_obs - factor_tmean
          
# Check the longitude and latitude
lat <- as.vector(attr(data_tasmax_obs, 'Variables')$dat1$lat)
lon <- as.vector(attr(data_tasmax_obs, 'Variables')$dat1$lon)

# lat_lon <- list()
# lat_lon$lat <- lat
# lat_lon$lon <- lon
# saveRDS(lat_lon, file=paste(path.intermediate_data, 'rdata/lat_lon_era5_eu.Rds', sep=""))

lat_mask <- InsertDim(InsertDim(lat,1,1),3,length(lon))
names(dim(lat_mask)) <- c("dat","latitude","longitude")

# Load monthly tmin and transform

variable <- variables[2]
path_obs_tasmin=paste0(path.downloaded_data,'era5land/monthly_mean/tasmin_f24h/$var$_$year$$month$.nc')

  data_tasmin_obs = Start(dataset = path_obs_tasmin,
                       var = variable,

                       year = paste0(years_obs),
                       month = 'all',

                       # lat = 'all',
                       # lon = values(lon_sel),

                       lat = values(list(lat_min,lat_max)), lat_reorder = Sort(decreasing = TRUE), lat_var = 'lat',
                       lon = values(list(lon_min,lon_max)), lon_reorder = CircularSort(-180,180), lon_var = 'lon',

                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       #lat_reorder = Sort(decreasing = F),
                       #lon_reorder = CircularSort(0, 360),
                       num_procs = n_procs_obs,
                       retrieve = retrieve)

  names(dim(data_tasmin_obs)) <- c("dat","member","sdate","time","latitude","longitude")

  # From K to degree C

  factor_tmean=273.15
  data_tasmin_obs <- data_tasmin_obs - factor_tmean

# Load monthly  prlr and transform

variable <- variables[3]
path_obs_pre=paste0(path.downloaded_data, 'era5land/monthly_mean/prlr_f1h/$var$_$year$$month$.nc')

data_pre_obs = Start(dataset = path_obs_pre,
                     var = variable,
                     
                     year = paste0(years_obs),
                     month = 'all',
                     
                     # lat = 'all',
                     # lon = values(lon_sel),
                     
                     lat = values(list(lat_min,lat_max)), lat_reorder = Sort(decreasing = TRUE), lat_var = 'lat',
                     lon = values(list(lon_min,lon_max)), lon_reorder = CircularSort(-180,180), lon_var = 'lon',
                     
                     synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                     return_vars = list(lat = 'dataset', lon = 'dataset'),
                     #lat_reorder = Sort(decreasing = F),
                     #lon_reorder = CircularSort(0, 360),
                     num_procs = n_procs_obs, 
                     retrieve = retrieve)

names(dim(data_pre_obs)) <- c("dat","member","sdate","time","latitude","longitude")

# From m.s-1 to mm.day-1
factor_pre=(60*60*24*1000) 
data_pre_obs <- data_pre_obs * (factor_pre*30.4375)

# Load monthly 2m dew point temperature and transform

variable <- variables[4]
path_obs_tdps=paste0(path.downloaded_data, 'era5land/monthly_mean/tdps_f1h/$var$_$year$$month$.nc')

data_tdps_obs = Start(dataset = path_obs_tdps,
                        var = variable,
                        
                        year = paste0(years_obs),
                        month = 'all',
                        
                        # lat = 'all',
                        # lon = values(lon_sel),
                        
                        lat = values(list(lat_min,lat_max)), lat_reorder = Sort(decreasing = TRUE), lat_var = 'lat',
                        lon = values(list(lon_min,lon_max)), lon_reorder = CircularSort(-180,180), lon_var = 'lon',
                        
                        synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        #lat_reorder = Sort(decreasing = F),
                        #lon_reorder = CircularSort(0, 360),
                        num_procs = n_procs_obs,
                        retrieve = retrieve)

names(dim(data_tdps_obs)) <- c("dat","member","sdate","time","latitude","longitude")

# From K to degree C

factor_tmean=273.15
data_tdps_obs <- data_tdps_obs - factor_tmean


# Load monthly sfcWind at 10m and transform to wind at 2m

variable <- variables[5]
path_obs_sfcWind=paste0(path.downloaded_data, 'era5land/monthly_mean/sfcWind_f1h/$var$_$year$$month$.nc')

data_sfcWind_obs = Start(dataset = path_obs_sfcWind,
                      var = variable,
                      
                      year = paste0(years_obs),
                      month = 'all',
                      
                      # lat = 'all',
                      # lon = values(lon_sel),
                      
                      lat = values(list(lat_min,lat_max)), lat_reorder = Sort(decreasing = TRUE), lat_var = 'lat',
                      lon = values(list(lon_min,lon_max)), lon_reorder = CircularSort(-180,180), lon_var = 'lon',
                      
                      synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                      return_vars = list(lat = 'dataset', lon = 'dataset'),
                      #lat_reorder = Sort(decreasing = F),
                      #lon_reorder = CircularSort(0, 360),
                      num_procs = n_procs_obs,
                      retrieve = retrieve)

names(dim(data_sfcWind_obs)) <- c("dat","member","sdate","time","latitude","longitude")

# FAO conversion formula (Allen et al.,): u2 <- uz * 4.87/ln((67.8*z) - 5.42)
# From 10-m to 2m windspeed -> u2 is approximately  = 0.7479511 * u10

factor_tmean=0.7479511 
data_sfcWind_obs <- data_sfcWind_obs * factor_tmean

# Load monthly surface pressure and transform 

variable <- variables[6]
path_obs_ps=paste0(path.downloaded_data, 'era5land/monthly_mean/ps_f1h/$var$_$year$$month$.nc')

data_ps_obs = Start(dataset = path_obs_ps,
                         var = variable,
                         
                         year = paste0(years_obs),
                         month = 'all',
                         
                         # lat = 'all',
                         # lon = values(lon_sel),
                         
                         lat = values(list(lat_min,lat_max)), lat_reorder = Sort(decreasing = TRUE), lat_var = 'lat',
                         lon = values(list(lon_min,lon_max)), lon_reorder = CircularSort(-180,180), lon_var = 'lon',
                         
                         synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                         return_vars = list(lat = 'dataset', lon = 'dataset'),
                         #lat_reorder = Sort(decreasing = F),
                         #lon_reorder = CircularSort(0, 360),
                         num_procs = n_procs_obs,
                         retrieve = retrieve)

names(dim(data_ps_obs)) <- c("dat","member","sdate","time","latitude","longitude")

# From P to kP surface pressure

factor_tmean=0.001 
data_ps_obs <- data_ps_obs * factor_tmean

# Load monthly Surface solar radiation downwards and transform 

variable <- variables[7]
path_obs_rsds=paste0(path.downloaded_data, 'era5land/monthly_mean/rsds_f1h/$var$_$year$$month$.nc')

data_rsds_obs = Start(dataset = path_obs_rsds,
                    var = variable,
                    
                    year = paste0(years_obs),
                    month = 'all',
                    
                    # lat = 'all',
                    # lon = values(lon_sel),
                    
                    lat = values(list(lat_min,lat_max)), lat_reorder = Sort(decreasing = TRUE), lat_var = 'lat',
                    lon = values(list(lon_min,lon_max)), lon_reorder = CircularSort(-180,180), lon_var = 'lon',
                    
                    synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                    return_vars = list(lat = 'dataset', lon = 'dataset'),
                    #lat_reorder = Sort(decreasing = F),
                    #lon_reorder = CircularSort(0, 360),
                    num_procs = n_procs_obs,
                    retrieve = retrieve)

names(dim(data_rsds_obs)) <- c("dat","member","sdate","time","latitude","longitude")

# From W/m2s toMJ/m2 day
# 1 day = 60s * 60min *24h = 86400s
# 1 W/m2 = 1 J/m2 s -> 294 J/m2 s x 86000s/1day = 25284000 J/m2 day
# 25284000 J/m2 day x 1MJ/1000000J = 25.284 MJ/m2 day

factor_tmean <- 0.0864
data_rsds_obs <- data_rsds_obs * factor_tmean


########## Handling NA's for PET estimation ###########

# data_tasmax_obs[which(is.na(data_tasmax_obs))] <- 0
# data_tasmin_obs[which(is.na(data_tasmin_obs))] <- 0
# data_pre_obs[which(is.na(data_pre_obs))] <- 0
# data_tdps_obs[which(is.na(data_tdps_obs))] <- 0
# data_sfcWind_obs[which(is.na(data_sfcWind_obs))] <- 0
# data_ps_obs[which(is.na(data_ps_obs))] <- 0
# data_rsds_obs[which(is.na(data_rsds_obs))] <- 0

################################################################################################
################################### PET6 estimation  ###########################################
################################################################################################

pet_method <- c('penman')

  ################################## P-PET: Model
  
  atomic_pet <- function(data_tasmax, data_tasmin, data_u2, data_rs, data_tdew, data_ps, lat_mask, pet_method){
    
      source("support_functions/Support_PET_estimation.R")
    
      ftime <- as.numeric(dim(data_tasmax)['time'])
      n_sdates <- as.numeric(dim(data_tasmax)['sdate'])
      lat <- lat_mask
      
      #print(lat)  
      # print(lon)
      
      if((sum(is.na(data_tasmax)) > 1) & (sum(is.na(data_tasmin)) > 1)  &  (sum(is.na(data_u2)) > 1) &  (sum(is.na(data_rs)) > 1) &   (sum(is.na(data_tdew)) > 1) &   (sum(is.na(data_ps)) > 1)){
        
        pet_estd <- array(NA, dim = c(ftime,n_sdates))
        
      } else {
        
        pet_estd <- PETx(tmax_mod=data_tasmax, tmin_mod=data_tasmin,
                         u2_mod=data_u2, rs_mod=data_rs, tdew_mod=data_tdew, ps_mod=data_ps,
                         pet_method=pet_method, lat=lat, na.rm = FALSE)
        
        pet_estd <- array(pet_estd, dim = c(ftime,n_sdates))
        
      }
      
      return(pet_estd)
      
  }
    

  # FAO-Penman approach
  
  print(pet_method)
  
  PET_estimated <- Apply(data=list(data_tasmax_obs,data_tasmin_obs,data_sfcWind_obs,data_rsds_obs,data_tdps_obs,data_ps_obs,lat_mask), 
                          target_dims = list(data_tasmax_obs=c('time','sdate'),data_tasmin_obs=c('time','sdate'),
                                     data_sfcWind_obs=c('time','sdate'),data_rsds_obs=c('time','sdate'),
                                     data_tdps_obs=c('time','sdate'),data_ps_obs=c('time','sdate'),
                                     lat_mask=c('dat')), 
                          pet_method=pet_method, 
                          output_dims = c('time','sdate'),
                          fun=atomic_pet, ncores = n_procs_obs)
  
  data_pet_obs <- drop(PET_estimated$output1)
  rm(PET_estimated)
  
  print('PET estimation completed...')
  
  saveRDS(data_pet_obs, file=paste(path.intermediate_data, 'rdata/data_pet_fao.Rds', sep=""))
  
################################################################################################
################################### P-PET Accumulation and SPEI computation ###########################################
################################################################################################

  data_pre_obs_x <- aperm(drop(data_pre_obs),c(2,1,3,4))

  diff_P_PET <- data_pre_obs_x - data_pet_obs
  rm(data_pre_obs_x)

# Atomic function for P-PET

atomic_accum <- function(data, accum_scale){

    ftime=as.numeric(dim(data)['time'])
    n_sdate= as.numeric(dim(data)['sdate'])
    scale <- accum_scale

    # Accumulation at different timescales
    data_vector <- as.vector(data)
    data_sum_x <- rollapply(data_vector, scale, sum)
    data_sum <- c(rep(NA, scale-1),data_sum_x)
    Accum6 <- array(data_sum, dim = c(ftime,n_sdate))

    return(Accum6)

}

# Accumulation
  
for (i in 1:12) {
    
    accum_scale <- i
    print(accum_scale)
    
    if(accum_scale == 1){
      
      data_accum_obs <- diff_P_PET[,2:as.numeric(dim(diff_P_PET)['sdate']),,]
      
    } else {
      accum <- Apply(data=list(diff_P_PET),
                     target_dims = list(diff_P_PET=c('time','sdate')),
                     accum_scale=accum_scale,
                     output_dims = c('time','sdate'),
                     fun=atomic_accum, ncores = n_procs_obs)
      data_accum_obs <- accum$output1[,2:as.numeric(dim(diff_P_PET)['sdate']),,]
      
    }
    
    saveRDS(data_accum_obs, file=paste(path.intermediate_data, 'accum/data_pet_fao_accum_',i,'.Rds', sep=""))
    rm(data_accum_obs)
    
}

# Standardization

regions <- 'EEA_subbasin'
n_procs_obs <- 4

for(i in 12){
  
  accum_scale <- i
  print(accum_scale)
  
  data_accum_obs <- readRDS(file=paste(path.intermediate_data, 'accum/data_pet_fao_accum_',i,'.Rds', sep=""))
  data_accum_obs <- data_accum_obs[,1:69,,] # analysis from 1950-2019
  ftime <- as.numeric(dim(data_accum_obs)['time'])
  n_sdates <- as.numeric(dim(data_accum_obs)['sdate'])
  
  mask <- readRDS(file=paste(path.intermediate_data, 'rdata/EEA_subbasin_mask_europe_era5land_COMBINED_REGIONS_1990-2019.Rds', sep=""))
  
  eur_mask <- mask$mask
  eur_mask[eur_mask == 0] <- NA
  nregion <- length(mask$region_code)
  
  names(dim(eur_mask)) <- c('country','latitude','longitude')
  
  # Redo dimension to apply mask
  
  spei_obs_masked <- array(NA, dim=c(nregion,ftime,n_sdates))
  
  for(count in 1:nregion){
    print(count)
    euro_mask_temp <- eur_mask[count,,]
    euro_mask_temp <- InsertDim(InsertDim(euro_mask_temp,1,ftime),2,n_sdates)
    
    b <- euro_mask_temp * data_accum_obs
    
    c <- s2dv::MeanDims(b,dims = c(3,4),na.rm = TRUE)
    
    spei_obs_masked[count,,] <- c
    
  }
  
  data_accum_obs_nuts2 <- aperm(spei_obs_masked, c(2,3,1))
  names(dim(data_accum_obs_nuts2)) <- c("time","sdate","country")
  
  
  saveRDS(data_accum_obs_nuts2, file=paste(path.intermediate_data, 'rdata/data_pet_fao_accum_',i,'_',regions,'_COMBINED_REGIONS_1990-2019.Rds', sep=""))
  
  
  # Standardization
  
  atomic_spei <- function(data){
    
    
    ftime <- as.numeric(dim(data)['time'])
    n_sdates <- as.numeric(dim(data)['sdate'])
    
    if((sum(is.na(data)) > 1)){
      
      spei_estd <- array(NA, dim = c(ftime,n_sdates))
      
    } else {
      
      data <- as.vector(data)
      
      spei_estd_x <- SPEI::spei(data = data, scale = 1, verbose = FALSE)
      spei_estd <- spei_estd_x$fitted
      spei_estd <- array(spei_estd, dim = c(ftime,n_sdates))
    }
    
    return(spei_estd)
    
  }
  
  spei6_cc <- Apply(data=list(data_accum_obs_nuts2),
                    target_dims = list(data_accum_obs=c('time','sdate')),
                    #output_dims = c('sdate','member'),
                    fun=atomic_spei, ncores = n_procs_obs)
  
  data_spei_obs_nuts2 <- spei6_cc$output1
  
  saveRDS(data_spei_obs_nuts2, file=paste(path.intermediate_data, 'rdata/spei/data_pet_fao_spei_',i,'_',regions,'_COMBINED_REGIONS_1950-2019.Rds', sep=""))
  
}
