

# This script aims to classify Europe into four regions: Northern, Southern, Easter and Western Europe depending on where the EEA river subbasin fall 

library(readxl)
library(sf, lib.loc = "/home/Earth/bsolaraj/R/x86_64-pc-linux-gnu-library/4.1")
library(ggplot2, lib.loc = "/home/Earth/bsolaraj/R/x86_64-pc-linux-gnu-library/4.1" )
library(ggspatial, lib.loc = "/home/Earth/bsolaraj/R/x86_64-pc-linux-gnu-library/4.1" )
library(dplyr, lib.loc = "/home/Earth/bsolaraj/R/x86_64-pc-linux-gnu-library/4.1")
library(colorspace, lib.loc = "/home/Earth/bsolaraj/R/x86_64-pc-linux-gnu-library/4.1")
library(RColorBrewer)

path.intermediate_data <- 'folder/'

regions_x <-  'EEA_riverbasin'

# Europe regions: UN Geoscheme
countries_southern <- c("AL", "AD", "BA", "HR", "CY", "GR", "VA", "IT", "MT", "ME", "MK", "PT", "SM", "RS", "SI", "ES", "EL")
countries_western <- c("AT", "BE", "FR", "DE", "LI", "LU", "MC", "NL", "CH")
countries_eastern <- c("BY", "BG", "CZ", "HU", "MD", "PL", "RO", "RU", "SK", "UA")
countries_northern <- c("DK", "EE", "FI", "IS", "IE", "LV", "LT", "NO", "SE", "UK")

# Read the country shapefile
shp_cc <-  st_read(paste0(path.intermediate_data, 'shapefiles/NUTS_RG_01M_2021_4326_LEVL_0.shp', crs = 4326))
regions <- st_read(paste0(path.intermediate_data, 'shapefiles/NUTS_RG_01M_2021_4326_LEVL_0.shp', crs = 4326))

regions$europe_part <- NA

nuts2_country_mapping <- list(
  southern = countries_southern,
  western = countries_western,
  eastern = countries_eastern,
  northern = countries_northern
)

# Iterate over each region
for (i in 1:nrow(regions)) {
  nuts2_code <- regions$NUTS_ID[i]
  
  # Check which country shapefile contains the NUTS2 code
  for (part in names(nuts2_country_mapping)) {
    if (nuts2_code %in% nuts2_country_mapping[[part]]) {
      regions$europe_part[i] <- part
      break
    }
  }
}

# River Sub-basin levels shapefile
river_subbasin <- st_read(paste(paste0(path.intermediate_data, 'exposure_data/WEI/SB_2022_v16/SB_2022_v16_3_CORRECTED_POLYGONS_FOR_ANALYSIS_COMBINED_REGIONS_1990-2019.shp', sep=""), crs = 4326))

# Perform spatial join to transform from river basin to sub river basin
regions_with_sb <- st_join(river_subbasin, regions, join = st_intersects)

# Group by entityCode and select the first Europe part
regions_europe_sb_unique <- regions_with_sb %>%
  group_by(SB_REF_V16) %>%
  slice(1) %>%
  select(SB_REF_V16, europe_part, geometry)

# Manual clasification of the regions of europe - Some corrections to correctly choose the regions

regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000403"] <- "southern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000402"] <- "southern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000110"] <- "southern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000136"] <- "southern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000618"] <- "southern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000439"] <- "southern"

regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000052"] <- "southern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000560"] <- "southern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000337"] <- "southern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000444"] <- "southern"

regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000011"] <- "western"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000147"] <- "western"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000419"] <- "western"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000071"] <- "western"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000172"] <- "western"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000122"] <- "western"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000424"] <- "western"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000070"] <- "western"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000445"] <- "western"

regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000068"] <- "eastern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000536"] <- "eastern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000299"] <- "eastern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000504"] <- "eastern"
regions_europe_sb_unique$europe_part[regions_europe_sb_unique$SB_REF_V16 == "WSB0000072"] <- "eastern"

saveRDS(regions_europe_sb_unique, file=paste(paste0(path.intermediate_data, 'rdata/exposure_data/data1_europe_regions_',regions_x,'.Rds', sep="")))

# Plotting general specifications

lat_min <- 34
lat_max <- 71.5
lon_min <- -15
lon_max <- 44

xmin<- lon_min
xmax<- lon_max
ymin<- lat_min
ymax<- lat_max

# Plot all Europe parts with different colors
x11()
colors <- c("western" = "blue", "southern" = "red", "northern" = "green", "eastern" = "purple")

ggplot() +
  geom_sf(data = regions_europe_sb_unique, aes(fill = europe_part)) +
  geom_sf(data = shp_cc, color = "lightgrey", fill = NA, size = 0.08, lwd = 0.3) +
  coord_sf(xlim = c(lon_min, lon_max), ylim = c(lat_min, lat_max)) +
  scale_fill_manual(values = colors) +
  theme_minimal()

x11()
ggplot() +
  geom_sf(data = regions, aes(fill = europe_part)) +
  geom_sf(data = shp_cc, color = "lightgrey", fill = NA, size = 0.08, lwd = 0.8) +
  coord_sf(xlim = c(lon_min, lon_max), ylim = c(lat_min, lat_max)) +
  scale_fill_manual(values = colors) +
  theme_minimal()
