<img src="figs/LANCET_EUROPE_LOGO.png" alt="drawing" width="270"/>  

<div align="justify">

This project is licensed under GNU General Public License version 3 ([GPL-3](https://www.gnu.org/licenses/gpl-3.0)|[View on GitLab](https://earth.bsc.es/gitlab/ghr/lcde-drought/-/blob/main/LICENSE))

[![License: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)    

</div>

# Indicator 1.2.2: Drought Indicator


## Introduction

Droughts are a major type of natural disaster, triggering a crisis with food security and public health. It is widely defined as a prolonged period of dry weather caused by a lack of rainfall. Such an extreme climate event can last from a few weeks to many decades and can affect from a few hundred to millions of square kilometres. We have assessed the drought condition using the Standardized Precipitation Evapotranspiration Index (SPEI). SPEI helps to detect, monitor and analyse the drought severity, duration, and extent globally. It is a standardised index with no units, where positive values of SPEI6 correspond to conditions of wet periods whereas negative values correspond to dry periods. 

## Data sources

| Variable | Source | Spatial Resolution | Temporal range used | 
| ------ | ------ | ------ | ------ |
| Monthly 2 meter maximum temperature | [ERA5-Land (monthly)](https://cds.climate.copernicus.eu/cdsapp#!/dataset/reanalysis-era5-land-monthly-means?tab=overview) | 0.1º x 0.1º | Jan 1950 - Dec 2021 |
| Monthly 2 meter minimum temperature | [ERA5-Land (monthly)](https://cds.climate.copernicus.eu/cdsapp#!/dataset/reanalysis-era5-land-monthly-means?tab=overview) | 0.1º x 0.1º | Jan 1950 - Dec 2021 |
| Monthly accumulated precipitation | [ERA5-Land (monthly)](https://cds.climate.copernicus.eu/cdsapp#!/dataset/reanalysis-era5-land-monthly-means?tab=overview) | 0.1º x 0.1º | Jan 1950 - Dec 2021 |
| Monthly 2m dewpoint temperature| [ERA5-Land (monthly)](https://cds.climate.copernicus.eu/cdsapp#!/dataset/reanalysis-era5-land-monthly-means?tab=overview) | 0.1º x 0.1º | Jan 1950 - Dec 2021 |
| Monthly 10m wind| [ERA5-Land (monthly)](https://cds.climate.copernicus.eu/cdsapp#!/dataset/reanalysis-era5-land-monthly-means?tab=overview) | 0.1º x 0.1º | Jan 1950 - Dec 2021 |
| Monthly surface pressure| [ERA5-Land (monthly)](https://cds.climate.copernicus.eu/cdsapp#!/dataset/reanalysis-era5-land-monthly-means?tab=overview) | 0.1º x 0.1º | Jan 1950 - Dec 2021 |
| Monthly surface solar radiation downwards| [ERA5-Land (monthly)](https://cds.climate.copernicus.eu/cdsapp#!/dataset/reanalysis-era5-land-monthly-means?tab=overview) | 0.1º x 0.1º | Jan 1950 - Dec 2021 |

## Methods

The computation of the SPEI6 index (as described in [Vicente-Serrano et al., 2010](https://journals.ametsoc.org/view/journals/clim/23/7/2009jcli2909.1.xml)) was done in two steps: accumulation and standardisation. First, the accumulation involves an estimation of monthly climatic water balance, which provides a measure of the water surplus or deficit for a specific month ‘i’ in the year ‘j’. It is estimated as follows:

d(i,j) = P(i,j)  – PET(i,j)

where, P is the precipitation and PET is the Potential Evapotranspiration. The computed d(i,j) for each month is then accumulated over a period of 6 months for the years 1951 to 2020 (A6(i,j)). 

A6(i,j) = d(i,j) + d(i-1,j)  + d(i-2,j) + … + d(i-5,j)

The standardisation step fits the A6(i,j) to a suitable parametric probability distribution and then transforms the data into a standardised series (with mean = 0 and standard deviation = 1), where the standardised value is referred to as SPEI6 (in which the number 6 corresponds to the total number of accumulated months). SPEI6-September, where the monthly climate water balance values of April to September are accumulated, is used to study the extended summer drought conditions and SPEI6-March that accumulates climate water balance values of October to March is used to study the extended winter drought condition. In this report, the spatially averaged results are presented at NUTS2 regions. When computing the index for NUTS 2 region, the accumulated climate water balance values were aggregated prior to standardisation step. 

The Hargreaves method is chosen for estimating PET in this assessment ([Hargreaves, 1994](https://ascelibrary.org/doi/10.1061/%28ASCE%290733-9437%281994%29120%3A6%281132%29)). The parameterization used for estimating PET using this approach is based on monthly maximum and minimum temperatures and latitude coordinates, the latter variable being used to calculate the maximum amount of sunshine duration. Following this, the three-parameter shifted log-logistic probability distribution function was used to build the distribution and the parameters were computed using the method of unbiased probability weighted moments. 


## Contents: 

- [0_EEA_subbasin_create_mask.R](https://earth.bsc.es/gitlab/ghr/lcde-drought/-/blob/2024/0_EEA_subbasin_create_mask.R): Script to create mask for the regions computing SPEI at EEA river subbasin level

- [1_compute_spei.R](https://earth.bsc.es/gitlab/ghr/lcde-drought/-/blob/2024/1_compute_spei.R): Script used to compute SPEI based on FAO penman monteith approach at EEA river subbasin level

- [2_organize_EEA_subbasin_WEI_data.R](https://earth.bsc.es/gitlab/ghr/lcde-drought/-/blob/2024/2_organize_EEA_subbasin_WEI_data.R): Script to organise the Quaterly WEI+ data at subbasin level that will be used for the analysis

- [3_EEA_subbasin_Classify_European_regions.R](https://earth.bsc.es/gitlab/ghr/lcde-drought/-/blob/2024/3_EEA_subbasin_Classify_European_regions.R): Script to to classify Europe into four regions: Northern, Southern, Easter and Western Europe depending on where the EEA river subbasin fall

- [4_EEA_subbasin_water_scaricity_during_severe_drought.R](https://earth.bsc.es/gitlab/ghr/lcde-drought/-/blob/2024/4_EEA_subbasin_water_scaricity_during_severe_drought.R): Script to provide the evolution of water scarcity condition during different drought condition

- [5_Plot_stacked_bar_two_different_period.R](https://earth.bsc.es/gitlab/ghr/lcde-drought/-/blob/2024/5_Plot_stacked_bar_two_different_period.R): Script to plot the water stress condition during different drought condition

- [6_Population_area_exposure_SPEI_severe_WEIplus_20.R](https://earth.bsc.es/gitlab/ghr/lcde-drought/-/blob/2024/6_Population_area_exposure_SPEI_severe_WEIplus_20.R): Script to organise the Quaterly WEI+ data at subbasin level that will be used for the analysis corresponding to water stress

- [6_Population_area_exposure_SPEI_severe_WEIplus_20.R](https://earth.bsc.es/gitlab/ghr/lcde-drought/-/blob/2024/6_Population_area_exposure_SPEI_severe_WEIplus_20.R): Script to organise the Quaterly WEI+ data at subbasin level that will be used for the analysis corresponding to high water stress

## Authors

Balakrishnan Solaraju-Murali, PhD
